
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import static java.util.concurrent.Executors.newFixedThreadPool;

public class ProfilingArrayParSort {

// test results on laptop i7 processor (two cores with hyperthreading)    
// Threads 1, parralel sort OFF with time: 15937
// Threads 1, parralel sort ON  with time: 9835
// Threads 2, parralel sort OFF with time: 8974
// Threads 3, parralel sort OFF with time: 7750
// Threads 10, parralel sort ON  with time: 7086
// Threads 4, parralel sort OFF with time: 7279
// Threads 2, parralel sort ON  with time: 7486
// Threads 3, parralel sort ON  with time: 7014
// Threads 4, parralel sort ON  with time: 6927
// Threads 10, parralel sort OFF with time: 5944
    
    public static void main(String[] args) {

        ProfilingArrayParSort tP = new ProfilingArrayParSort();
        List<String> results = new ArrayList<>();
        
        results.add(tP.test(1,false));
        results.add(tP.test(1,true));
        results.add(tP.test(2,false));
        results.add(tP.test(3,false));
        results.add(tP.test(10,true));
        results.add(tP.test(4,false));
        results.add(tP.test(2,true));
        results.add(tP.test(3,true));
        results.add(tP.test(4,true));
        results.add(tP.test(10,false));
        
        for (String result : results) {
            System.out.println(result);
        }

    }

    ExecutorService threadPool;
    boolean parSort;

    public String test(int numberOfThreads, boolean parallelSort) {

        parSort = parallelSort;
        threadPool = newFixedThreadPool(numberOfThreads);
        List<Future> futures = new ArrayList<>();

        long time = System.currentTimeMillis();        
        
        for (int i = 1; i <= 10; i++) {
            Callable<String> aCallable = new CallableImpl(i);
            Future<String> submit = threadPool.submit(aCallable);
            futures.add(submit);
        }

        try {
            for (Future future : futures) {
                System.out.println(future.get());
            }
        } catch (Exception ex) {
            System.out.println("Not ready ?? ...");
        }

        threadPool.shutdown(); // shutdown the pool.

        return("Threads " + numberOfThreads + ", parralel sort " + (parSort?"ON ":"OFF") + " with time: " + (System.currentTimeMillis() - time));

    }

    private class CallableImpl implements Callable<String> {

        String name;
        boolean sortTweak;

        public CallableImpl(int number) {
            name = "Callable " + number;
        }

        @Override
        public String call() {
            Random random = new Random();
            for (int i = 0; i < 2; i++) {
                System.out.println(name + " is doing something");
                final int size = 5000000;
                double[] arr = new double[size];
                sortTweak = false;

                for (int j = 0; j < size; j++) {
                    arr[j] = random.nextDouble();
                }
                System.out.println(name + " [" + i + "] Max: " + getMax(arr));
                System.out.println(name + " [" + i + "] Min: " + getMin(arr));
                System.out.println(name + " [" + i + "] Avg: " + getAvg(arr));
                System.out.println(name + " [" + i + "] Sum: " + getSum(arr));
                System.out.println(name + " [" + i + "] Med: " + getMed(arr));
            }
            return name + " is done !";
        }

        private void sortTweakIt(double[] arr) {
            if (!sortTweak) {
                System.out.println("Tweaking ... ");

                if (parSort) {
                    Arrays.parallelSort(arr);
                } else {
                    Arrays.sort(arr);
                }
                sortTweak = true;
            }
        }

        private double getMax(double[] arr) {
            sortTweakIt(arr);
            return arr[arr.length - 1];
        }

        private double getMin(double[] arr) {
            sortTweakIt(arr);
            return arr[0];
        }

        private double getSum(double[] arr) {
            double sum = 0;
            for (Double d : arr) {
                sum += d;
            }
            return sum;
        }

        private double getAvg(double[] arr) {
            return getSum(arr) / arr.length;
        }

        private double getMed(double[] arr) {
            sortTweakIt(arr);
            return arr[arr.length / 2];
        }

    }

}
