
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import static java.util.concurrent.Executors.newFixedThreadPool;

public class ProfilingListBetter {

// test results on laptop i7 processor (two cores with hyperthreading)    
// Threads 1 with time: 10192
// Threads 2 with time: 6335
// Threads 3 with time: 5326
// Threads 4 with time: 4760
// Threads 5 with time: 4539
// Threads 10 with time: 4374
    public static void main(String[] args) {

        ProfilingListBetter tP = new ProfilingListBetter();
        List<String> results = new ArrayList<>();

        results.add(tP.test(1));
        results.add(tP.test(2));
        results.add(tP.test(3));
        results.add(tP.test(4));
        results.add(tP.test(5));
        results.add(tP.test(10));

        for (String result : results) {
            System.out.println(result);
        }

    }

    ExecutorService threadPool;

    public String test(int numberOfThreads) {

        threadPool = newFixedThreadPool(numberOfThreads);
        List<Future> futures = new ArrayList<>();
        long time = System.currentTimeMillis();

        for (int i = 1; i <= 10; i++) {
            Callable<String> aCallable = new CallableImpl(i);
            Future<String> submit = threadPool.submit(aCallable);
            futures.add(submit);
        }

        try {
            for (Future future : futures) {
                System.out.println(future.get());
            }
        } catch (Exception ex) {
            System.out.println("Not ready ?? ...");
        }

        threadPool.shutdown(); // shutdown the pool.

        return ("Threads " + numberOfThreads + " with time: " + (System.currentTimeMillis() - time));

    }

    private class CallableImpl implements Callable<String> {

        String name;
        boolean sortTweak;

        public CallableImpl(int number) {
            name = "Callable " + number;
        }

        @Override
        public String call() {
            for (int i = 0; i < 2; i++) {
                System.out.println(name + " is doing something");
                int size = 1000000;
                //List<Double> arr = new LinkedList<>(); // first version
                //List<Double> arr = new ArrayList<>(); // so, is it wrong list?
                List<Double> arr = new ArrayList<>(size); // hmmm, maybe list resize on add??
                sortTweak = false;
                generate(size, arr);
                System.out.println(name + " [" + i + "] Max: " + getMax(arr));
                System.out.println(name + " [" + i + "] Min: " + getMin(arr));
                System.out.println(name + " [" + i + "] Avg: " + getAvg(arr));
                System.out.println(name + " [" + i + "] Sum: " + getSum(arr));
                System.out.println(name + " [" + i + "] Med: " + getMed(arr));
            }
            return name + " is done !";
        }

        private void generate(int size, List<Double> arr) {
            Random random = new Random();
            for (int j = 0; j < size; j++) {
                arr.add(random.nextDouble());
            }
        }

        private void sortTweakIt(List<Double> arr) {
            if (!sortTweak) {
                System.out.println("Tweaking ... ");
                Collections.sort(arr);
                sortTweak = true;
            }
        }

        private Double getMax(List<Double> arr) {
            sortTweakIt(arr);
            return arr.get(arr.size() - 1);
        }

        private Double getMin(List<Double> arr) {
            sortTweakIt(arr);
            return arr.get(0);
        }

        private Double getSum(List<Double> arr) {
            double sum = 0;
            for (Double d : arr) {
                sum += d;
            }
            return sum;
        }

        private Double getAvg(List<Double> arr) {
            return getSum(arr) / arr.size();
        }

        private Double getMed(List<Double> arr) {
            sortTweakIt(arr);
            return arr.get(arr.size() / 2);
        }

    }

}
